"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const common_1 = require("./common");
const maxByAsync_1 = require("./maxByAsync");
const equatable_1 = require("equatable");
/**
 * Gets the maximum element of an [[AsyncQueryable]], optionally comparing elements using the supplied callback.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param comparer An optional callback used to compare two elements.
 * @category Scalar
 */
function maxAsync(source, comparer = equatable_1.Comparer.defaultComparer) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (typeof comparer === "function")
            comparer = equatable_1.Comparer.create(comparer);
        internal_1.assert.mustBeAsyncQueryable(source, "source");
        internal_1.assert.mustBeComparer(comparer, "comparer");
        return maxByAsync_1.maxByAsync(source, common_1.identity, comparer);
    });
}
exports.maxAsync = maxAsync;
//# sourceMappingURL=maxAsync.js.map