"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const collections_1 = require("../collections");
const collections_2 = require("equatable/collections");
function symmetricDifferenceBy(left, right, keySelector, keyEqualer) {
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeEqualerOrUndefined(keyEqualer, "keyEqualer");
    return internal_1.FlowHierarchy(new SymmetricDifferenceByIterable(internal_1.ToIterable(left), internal_1.ToIterable(right), keySelector, keyEqualer), left, right);
}
exports.symmetricDifferenceBy = symmetricDifferenceBy;
let SymmetricDifferenceByIterable = class SymmetricDifferenceByIterable {
    constructor(left, right, keySelector, keyEqualer) {
        this._left = left;
        this._right = right;
        this._keySelector = keySelector;
        this._keyEqualer = keyEqualer;
    }
    *[Symbol.iterator]() {
        const keySelector = this._keySelector;
        const rightKeys = this._keyEqualer ? new collections_2.HashSet(this._keyEqualer) : new collections_1.Set();
        const right = this._keyEqualer ? new collections_2.HashMap(this._keyEqualer) : new Map();
        for (const element of this._right) {
            const key = keySelector(element);
            if (internal_1.TryAdd(rightKeys, key)) {
                right.set(key, element);
            }
        }
        const set = this._keyEqualer ? new collections_2.HashSet(this._keyEqualer) : new collections_1.Set();
        for (const element of this._left) {
            const key = keySelector(element);
            if (internal_1.TryAdd(set, key) && !right.has(key)) {
                yield element;
            }
        }
        for (const [key, element] of right) {
            if (internal_1.TryAdd(set, key)) {
                yield element;
            }
        }
    }
};
SymmetricDifferenceByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("SymmetricDifferenceByIterable")
], SymmetricDifferenceByIterable);
//# sourceMappingURL=symmetricDifferenceBy.js.map