"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const common_1 = require("./common");
function toObject(source, prototype = Object.prototype, keySelector, elementSelector = common_1.identity, descriptorSelector = internal_1.MakeDescriptor) {
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeObjectOrNull(prototype, "prototype");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(descriptorSelector, "descriptorSelector");
    const obj = prototype === Object.prototype ? {} : Object.create(prototype);
    for (const item of internal_1.ToIterable(source)) {
        const key = keySelector(item);
        const element = elementSelector(item);
        const descriptor = descriptorSelector(key, element);
        Object.defineProperty(obj, key, descriptor);
    }
    return obj;
}
exports.toObject = toObject;
//# sourceMappingURL=toObject.js.map