/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
import { AsyncQueryable } from "../types";
import { Equaler } from 'equatable';
import { HashSet } from 'equatable/collections';
/**
 * Creates a Set for the elements of the Query.
 *
 * @param source An [[AsyncQueryable]] object.
 * @category Scalar
 */
export declare function toSetAsync<T>(source: AsyncQueryable<T>): Promise<Set<T>>;
/**
 * Creates a Set for the elements of the Query.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param equaler An [[Equaler]] object used to compare equality.
 * @category Scalar
 */
export declare function toSetAsync<T>(source: AsyncQueryable<T>, equaler: Equaler<T>): Promise<HashSet<T>>;
/**
 * Creates a Set for the elements of the Query.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param elementSelector A callback that selects a value for each element.
 * @category Scalar
 */
export declare function toSetAsync<T, V>(source: AsyncQueryable<T>, elementSelector: (element: T) => V | PromiseLike<V>): Promise<Set<V>>;
/**
 * Creates a Set for the elements of the Query.
 *
 * @param source An [[AsyncQueryable]] object.
 * @param elementSelector A callback that selects a value for each element.
 * @param equaler An [[Equaler]] object used to compare equality.
 * @category Scalar
 */
export declare function toSetAsync<T, V>(source: AsyncQueryable<T>, elementSelector: (element: T) => V | PromiseLike<V>, equaler: Equaler<V>): Promise<HashSet<V>>;
//# sourceMappingURL=toSetAsync.d.ts.map