"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var equatable_1 = require("equatable");
/**
 * Gets the minimum element of a [[Queryable]], optionally comparing the keys of each element using the supplied callback.
 *
 * @param source A [[Queryable]] object.
 * @param keySelector A callback used to choose the key to compare.
 * @param keyComparer An optional callback used to compare the keys.
 * @category Scalar
 */
function minBy(source, keySelector, keyComparer) {
    var e_1, _a;
    if (keyComparer === void 0) { keyComparer = equatable_1.Comparer.defaultComparer; }
    if (typeof keyComparer === "function")
        keyComparer = equatable_1.Comparer.create(keyComparer);
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeComparer(keyComparer, "keyComparer");
    var hasResult = false;
    var result;
    var resultKey;
    try {
        for (var _b = tslib_1.__values(internal_1.ToIterable(source)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var element = _c.value;
            var key = keySelector(element);
            if (!hasResult) {
                result = element;
                resultKey = key;
                hasResult = true;
            }
            else if (keyComparer.compare(key, resultKey) < 0) {
                result = element;
                resultKey = key;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return result;
}
exports.minBy = minBy;
//# sourceMappingURL=minBy.js.map