"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var common_1 = require("./common");
function toObject(source, prototype, keySelector, elementSelector, descriptorSelector) {
    var e_1, _a;
    if (prototype === void 0) { prototype = Object.prototype; }
    if (elementSelector === void 0) { elementSelector = common_1.identity; }
    if (descriptorSelector === void 0) { descriptorSelector = internal_1.MakeDescriptor; }
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeObjectOrNull(prototype, "prototype");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeFunction(elementSelector, "elementSelector");
    internal_1.assert.mustBeFunction(descriptorSelector, "descriptorSelector");
    var obj = prototype === Object.prototype ? {} : Object.create(prototype);
    try {
        for (var _b = tslib_1.__values(internal_1.ToIterable(source)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var item = _c.value;
            var key = keySelector(item);
            var element = elementSelector(item);
            var descriptor = descriptorSelector(key, element);
            Object.defineProperty(obj, key, descriptor);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return obj;
}
exports.toObject = toObject;
//# sourceMappingURL=toObject.js.map