"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const collections_1 = require("../collections");
const collections_2 = require("equatable/collections");
function distinctByAsync(source, keySelector, keyEqualer) {
    internal_1.assert.mustBeAsyncQueryable(source, "source");
    internal_1.assert.mustBeFunction(keySelector, "keySelector");
    internal_1.assert.mustBeEqualerOrUndefined(keyEqualer, "keyEqualer");
    return internal_1.FlowHierarchy(new AsyncDistinctByIterable(internal_1.ToPossiblyAsyncIterable(source), keySelector, keyEqualer), source);
}
exports.distinctByAsync = distinctByAsync;
let AsyncDistinctByIterable = class AsyncDistinctByIterable {
    constructor(source, keySelector, keyEqualer) {
        this._source = source;
        this._keySelector = keySelector;
        this._keyEqualer = keyEqualer;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a;
            const set = this._keyEqualer ? new collections_2.HashSet(this._keyEqualer) : new collections_1.Set();
            const selector = this._keySelector;
            try {
                for (var _b = tslib_1.__asyncValues(this._source), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                    const element = _c.value;
                    const key = selector(element);
                    if (!set.has(key)) {
                        set.add(key);
                        yield yield tslib_1.__await(element);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
};
AsyncDistinctByIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncDistinctByIterable")
], AsyncDistinctByIterable);
//# sourceMappingURL=distinctByAsync.js.map