"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const internal_1 = require("../internal");
const common_1 = require("./common");
const minBy_1 = require("./minBy");
const equatable_1 = require("equatable");
/**
 * Gets the minimum element of a [[Queryable]], optionally comparing elements using the supplied callback.
 *
 * @param source A [[Queryable]] object.
 * @param comparer An optional callback used to compare two elements.
 * @category Scalar
 */
function min(source, comparer = equatable_1.Comparer.defaultComparer) {
    if (typeof comparer === "function")
        comparer = equatable_1.Comparer.create(comparer);
    internal_1.assert.mustBeQueryable(source, "source");
    internal_1.assert.mustBeComparer(comparer, "comparer");
    return minBy_1.minBy(source, common_1.identity, comparer);
}
exports.min = min;
//# sourceMappingURL=min.js.map